# R script for performing conditonal uniform and permutation tests
# with the package 'sna'.
# 
# The "Faux highschool" data are courtesy of Jim Moody. The data
# set is simulated, the network is symmetric. The simulation was
# inspired by exponential random graph modelling of the Add Health
# data sets.
# 
# Bearman, P.S.; Moody, J.; Stovel, K.; & Thalji, L. (2004). 
#     Social and sexual networks: The national longitudinal study
#     of adolescent health. In Morris, Martina (Ed.), Network
#     Epidemiology: A Handbook for Survey Design and Data Collection
#     (pp. 201-237). New York, NY: Oxford University Press.).
# 
# 
# Overview of topics:
# ===================
# a) Getting ready: directory settings, library, data entry
# b) Network inference assuming conditional independence
# c) Network inference using permutation tests
# 
#


# ========================================================= 
# a) Getting ready: directory settings, library, data entry
# =========================================================
# 
# This repeats steps from last week's script Lab1a.R, but for
# a different data set.

setwd("<put here the path to the folder where your data are>")   

# sets the working directory to the location where the 
# NOTE: YOU NEED TO CHANGE THIS TO WHERE YOU STORED THESE FILES!
# Note that R does not like Windows single backslashes "\" in directory paths.
# Replace them either by forward slashes "/" or double backslashes "\\" as in the example.

list.files()  

# lists the files in the directory.

# To make the 'sna' (and 'network') commands available, add them to the current
# dictionary by typing:

library(network)
library(sna)

# Read the faux high school data files:
			
fauxhs <- read.csv("data.fauxhigh.csv")
fauxhs.edges <- read.csv("edges.fauxhigh.csv",header=TRUE)
fauxhs.net <- network.initialize(n=120,directed=FALSE)
fauxhs.net <- network.edgelist(fauxhs.edges,fauxhs.net)

# Put grade, gender and race onto the network object:

fauxhs.net %v% "grade" <- fauxhs$grade  
fauxhs.net %v% "gender" <- as.character(fauxhs$gender) 
fauxhs.net %v% "race" <- as.character(fauxhs$race)

# What objects do we have now?

objects()    # an alias of this command is the shorter "ls()"

# Two of the three variables here are redundant - why not be sparse
# in memory use and drop them?

remove(fauxhs,fauxhs.edges)
ls()

# now only "fauxhs.net" remains, which contains the 

# 
# 
# ======================================================
# b) Network inference assuming conditional independence
# ======================================================
#
# So far we have seen how to calculate graph level indicies like density
# and transitivity. But how do we know if the values are large, relative
# to some baseline? 
# 
# One approach is to generate a distribution of simulated, conditioned
# random networks, distill from this a distribution of the statistic of
# interest, and see where the observed value of the statistic is located
# in this distribution.

# Starting with a density test:

gden(fauxhs.net)
cug.test(fauxhs.net,"gden",cmode="size") 

# The above compares the observed density to the density of random networks
# with density 0.5 of the right size.

# Next, now transitivity 

gtrans(fauxhs.net)
cug.test(fauxhs.net,"gtrans",cmode=c("edges"))

# The above conditions the random comparison on the number of edges.
# So, relative to a random network with the same amount of edges
# there would appear to be  high clustering in the network.
#
# Consult the following to find out how to condition on the whole dyad census:

?cug.test

# For more general triad-based testing, let's again consider the triad census:

triad.census(fauxhs.net)

# For obtaining a flexible way of testing, make a little function which we can
# then use in cug.test... Take the linear combination idea based on weighting
# the triad census used in the earleir script, but now make a function out of it:

triad.func <- function(net,triad.weight){sum(triad.weight*triad.census(net))} 

# In the above function, "triad.weight" must be a vector with 16 weight entries
# which identifies which triads in the census to sum up how often.

transitivity.weight <- c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1) 

# The example above identifies the full triangles as to-be-tested statistic.
# Note that we here have a symmetric network, so this is the only 'transitive'
# triad type!

# Are there more full triangles in the observed network than we would expect, given
# the number of edges?

cug.test(fauxhs.net,"triad.func",
	FUN.args=list(triad.weight=transitivity.weight),
	cmode="edges",reps=100)

# What about two-paths between unconnected actors?

twopath.weight <- c(,,,,,,,,,,,,,,,)
# (fill in the correct weights during the course!)

cug.test(fauxhs.net,"triad.func",
	FUN.args=list(triad.weight=twopath.weight),
	cmode="edges",reps=100)

# 
# 
# 
# ============================================
# c) Network inference using permutation tests
# ============================================
#
# Now we are going to take a slightly different approach and use linear
# regression to answer questions about the features of the network.
# Specifically, we want to know if the students in Faux High School are
# more likely to have friends in the same grade.
# So, is homophily on grade strong? 

# First we need a matrix showing if every pair i, j match on grade or not:

grade <- get.vertex.attribute(fauxhs.net,"grade") 

# The above gets the grade of each student, as you can see by calling

grade

# Now generate the matrix of grade matching (yes or no): 

grade.matrix <- 1*outer(grade,grade,"==")
grade.matrix[1:10,1:10]

# (Try out what happens if you omit the "1*" in the matrix generation!)
# (Yes, there are many other ways in R to get the same matrix.)
#
# Now let us run a network regression:

dyad.ind <- netlogit(fauxhs.net,grade.matrix,mode="graph",
	nullhyp="classical") 

# The above performs a dyadic independent logistic regression.
# The null hypothesis is that networks are random with the right number of edges
# (so the dyads are independent).

# Consult the following to find out other options of this function:

?netlogit

# By the way: we can also run this model in a simple logistic regression 
# framework, which we do in the following "excursion":

	# First, we need to create the dependent variable "whether or not there 
	# is a tie in each dyad", as a long string of one's and zeros.

	ties <- as.matrix(fauxhs.net) 

	# Omit diagonal entries:

	diag(ties) <- NA
	ties

	# Because Faux High School friendship is a symmetric network, the (upper right) half
	# of the matrix is fully informative - let's drop the other (lower left) half:

	ties <- ties[upper.tri(ties)]
	ties

	# Do the same for the grade match predictor variable:

	grade.match <- grade.matrix
	grade.match <- grade.match[upper.tri(grade.match)] 

	# The above strings out the grade data so we can correlate having a tie 
	# with matching on grade or not.

	grade.match
	dyad.data <- data.frame(ties,grade.match)                          

	# Let's take a look at the data:

	dyad.data[1:10,]

	summary(glm(ties~grade.match,family="binomial",data=dyad.data))

	# The above specifies a model predicting the presence of a tie as
	# a function of matching on grade; the baseline is a random network
	# with the right number of edges. We should obtain the same results
	# as above.

# Okay, after this excursion to 'regular' statistics, back to "sna"! 

# Let us now do a permutation test with the same regression model!
#
# So, we use the same analysis, but we using a different null hypothesis.
#
# Permutation tests randomly permute the actor labels, rerun the regression
# each time, and compare the regression coefficients obtained from the observed
# data to the distribution of coefficients obtained for the permuted datasets.

dyad.qap <- netlogit(fauxhs.net,grade.matrix,mode="graph",
	nullhyp="qapy",reps=100) 

# The above estimates the same parameters as the 'glm' model above
# but it obtains  standard error equivalents using the permutation test.

summary(dyad.qap)                                                               

# It would appear that homophily on grade is strong in the network.

# ==============================================================================
